"use strict";
/* eslint-disable canonical/filename-match-regex */
Object.defineProperty(exports, "__esModule", { value: true });
exports.TurboWatcher = void 0;
const hashFile_1 = require("../hashFile");
const isFSWatcherAvailable_1 = require("../isFSWatcherAvailable");
const Logger_1 = require("../Logger");
const ChokidarWatcher_1 = require("./ChokidarWatcher");
const FileWatchingBackend_1 = require("./FileWatchingBackend");
const FSWatcher_1 = require("./FSWatcher");
const log = Logger_1.Logger.child({
    namespace: 'TurboWatcher',
});
// eslint-disable-next-line node/no-process-env
const { TURBOWATCH_EXPERIMENTAL_FILE_HASH } = process.env;
class TurboWatcher extends FileWatchingBackend_1.FileWatchingBackend {
    constructor(project) {
        super();
        if (TURBOWATCH_EXPERIMENTAL_FILE_HASH === 'true') {
            log.warn('using experimental file hashing mechanism');
        }
        if ((0, isFSWatcherAvailable_1.isFSWatcherAvailable)()) {
            log.info('using native FSWatcher');
            this.backend = new FSWatcher_1.FSWatcher(project);
        }
        else {
            log.info('using native ChokidarWatcher');
            this.backend = new ChokidarWatcher_1.ChokidarWatcher(project);
        }
        this.backend.on('ready', () => {
            this.emit('ready');
        });
        this.backend.on('change', (event) => {
            // This implementation is somewhat problematic, because it hashes all the files,
            // even if those files are not matched by the triggers. We should probably consider
            // moving this logic up to watch.ts, and only hash the files that are matched.
            if (TURBOWATCH_EXPERIMENTAL_FILE_HASH === 'true') {
                // eslint-disable-next-line promise/prefer-await-to-then
                (0, hashFile_1.hashFile)(event.filename).then((hash) => {
                    this.emit('change', {
                        ...event,
                        hash,
                    });
                });
            }
            else {
                this.emit('change', event);
            }
        });
    }
    close() {
        return this.backend.close();
    }
}
exports.TurboWatcher = TurboWatcher;
//# sourceMappingURL=TurboWatcher.js.map